#!/usr/bin/lua

--[[ Program a sabldhiv script kimenetenek feldolgozasara. Szetvalogatja a szocikkeket 3 csoportra.
	Bemenet:
		- az elozo lekerdezes datuma eeee-ho-nn.oo:pp:ss alakban
		- a datumos szocikklista (kemmol.mol). A sorok alakja: szocikk|eeee-ho-nn.oo:pp:ss
		- regi osszegkepletek (hufo.egybe)
		- regi idostatisztika (krealasi es modositasi idok)
		- regi SMILES-ok
	Eredmeny:
		- sabldhiv.mod:  modosult (ujra lekerdezendo) szocikkek
		- sabldhiv.egybe: nem modosult szocikkek osszegkepletei
		- sabldhiv.ido: nem modosult szocikkek letrehozasi es modositasi idoi
		- sabldhiv.smiles: nem modosult SMILES-ok
]]

			-- Helyi fuggvenyek

function fnyit(fnev)		-- irasra nyit meg egy fajlt, visszaadja a file pointert
local fp = io.open(fnev,"w")
if not fp
then	io.stderr:write('Nem tudom irasra megnyitni: '..fnev..'\n')
	os.exit(2)
end
return fp
end

local hivas = 'Hivas: '..arg[0]..' datum listafile kezilista regi_egybefile regi_idofile regi_SMILES_file\n'

function fnyitolv(fnev)		-- megnezi, letezik-e (olvashato-e) egy fajl. Ha nem, kilep a programbol.
local fp
if fnev then fp = io.open(fnev) end
if not fp
then	io.stderr:write(hivas..'Nincs ilyen file: '..(fnev or '')..'\n')
	os.exit(1)
end
io.close(fp)	-- hulye lua!
return
end

function cikkcim(sor,szepkar)		-- visszaadja a sorban az elso | elotti szoveget vagy nil-t
local ret = string.match(sor,"^[^"..szepkar.."]+")
if ret then ret = string.gsub(ret,' ','_') end
return ret
end

function cikkertek(sor,szepkar)		-- visszaadja az elso | utani szoveget vagy nil-t, ha nincs |
local ret = string.match(sor,szepkar..".+$")
if ret then ret = string.sub(ret,2) end
return ret
end

function beolv(fnev,lista,szepkar)	-- egy fajl beolvasasa a tarba asszociativ tombbe
fnyitolv(fnev)
for line in io.lines(fnev) do
	local cim = cikkcim(line,szepkar)
	local dat = cikkertek(line,szepkar)
	if cim  then
		--if dat then lista[cim] = dat else lista[cim] = '' end
		if dat then lista[cim] = dat else lista[cim] = true end
	end
end
end

			-- Parameterellenorzes

local datform = '^[2-9]%d%d%d[-][0-1]%d[-][0-3]%d[.][0-2][0-9]:[0-5][0-9]:[0-5][0-9]'	-- $ szandekosan nincs a vegen
local space = '     '
local minta = string.match(arg[1] or '',datform)

if not arg[1] or arg[1] ~= minta
then	io.stderr:write(hivas..'A datum alakja: eeee-ho-nn.oo:pp:ss'..space..(arg[1] or '')..space..(minta or '')..'\n')
	os.exit(1)
end
--print('Jo a datum')

kemmol = {}
beolv(arg[2],kemmol,'|')	-- datumos szocikklista
fnyitolv(arg[3])		-- regi osszegkepletek
fnyitolv(arg[4])		-- idostatisztika
smiles   = {}			-- SMILES
beolv(arg[5],smiles,'|')

fpmod    = fnyit("sabldhiv.mod")		-- arg[1]-nel ujabb szocikkek
fpegybe  = fnyit("sabldhiv.egybe")		-- a regibol masolt kepletek
fpido    = fnyit("sabldhiv.ido")		-- a regibol masolt letrehozasi es modositasi idok
fpsmiles = fnyit("sabldhiv.smiles")		-- a regibol masolt SMILES-ok


			-- Most jon a lenyeg

kiirt = {}		-- a modosultak kozott kiirt szocikkek (a regiek kozul)
for line in io.lines(arg[3]) do		-- beolvassuk az elozo hufo.egybe-t
	local cim = cikkcim(line,'#')
	if kemmol[cim] then		-- kemmol-ban van az osszes szocikk, amivel foglalkozni kell
		if kemmol[cim] <= arg[1] then
			fpegybe:write(line..'\n')	-- regebbi, mint a futas: regi hufo.egybe-bol lehet venni
			kiirt[cim] = true		-- eltaroljuk, nehogy ketszer irjuk ki a lekerdezendok koze
		end
	end
end
fpegybe:close()		-- a nem modosult szocikkek keszen vannak

for cim in pairs(kemmol) do	-- kiirjuk a modosult szocikkeket
	if not kiirt[cim] then fpmod:write(cim..'\n') end
end
fpmod:close()		-- a modosult osszegkepletek is keszek

			-- az osszegkepletekkel keszen vagyunk
kiirt = nil		-- a SMILES-okhoz meg kelleni fog

-- fpido kovetkezik
for line in io.lines(arg[4]) do		-- beolvassuk az elozo kemcre.ido-t
	local cim = cikkcim(line,'|')
	if kemmol[cim] then		-- kemmol-ban van az osszes szocikk, amivel foglalkozni kell
		if kemmol[cim] <= arg[1] then
			fpido:write(line..'\n')	-- regebbi, mint a futas: regi kemcre.ido-bol lehet venni
		end
	end
end
fpido:close()		-- az idok keszen vannak
